/*
 *  UltimateIRCd - an Internet Relay Chat Daemon, include/struct.h
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: struct.h,v 1.12 2002/04/06 00:28:34 shadowmaster Exp $
 *
 */

#ifndef	__struct_include__
#define __struct_include__

#include "config.h"
#if !defined(CONFIG_H_LEVEL_12)
#error Incorrect config.h for this revision of ircd.
#endif

#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#if defined( HAVE_STDDEF_H )
#include <stddef.h>
#endif
#ifdef ORATIMING
#include <sys/time.h>
#endif

#ifdef USE_SYSLOG
#include <syslog.h>
#if defined( HAVE_SYS_SYSLOG_H )
#include <sys/syslog.h>
#endif
#endif

#define REPORT_DO_DNS_		":%s NOTICE AUTH :*** Looking up your hostname...\r\n"
#define REPORT_FIN_DNS_		":%s NOTICE AUTH :*** Found your hostname\r\n"
#define REPORT_FIN_DNSC_	":%s NOTICE AUTH :*** Found your hostname, cached\r\n"
#define REPORT_FAIL_DNS_	":%s NOTICE AUTH :*** Couldn't look up your hostname\r\n"
#define REPORT_DO_ID_		":%s NOTICE AUTH :*** Checking Ident\r\n"
#define REPORT_FIN_ID_		":%s NOTICE AUTH :*** Got Ident response\r\n"
#define REPORT_FAIL_ID_		":%s NOTICE AUTH :*** No Ident response\r\n"

#define REPORT_DO_PROXY_ 	":%s NOTICE AUTH :*** Checking for insecure socks/proxy server...\r\n"
#define	REPORT_NO_PROXY_ 	":%s NOTICE AUTH :*** No socks/proxy server found (good)\r\n"
#define REPORT_GOOD_PROXY_ 	":%s NOTICE AUTH :*** Secure socks/proxy server found\r\n"
#define REPORT_PROXYEXEMPT_ 	":%s NOTICE AUTH :*** Exempt line found, Skipping socks/proxy check...\r\n"

extern char REPORT_DO_DNS[256], REPORT_FIN_DNS[256], REPORT_FIN_DNSC[256],
  REPORT_FAIL_DNS[256], REPORT_DO_ID[256], REPORT_FIN_ID[256],
  REPORT_FAIL_ID[256], REPORT_DO_PROXY[256], REPORT_NO_PROXY[256],
  REPORT_GOOD_PROXY[256], REPORT_PROXYEXEMPT[256];

#include "hash.h"

typedef struct ConfItem aConfItem;
typedef struct Client aClient;
typedef struct Channel aChannel;
typedef struct User anUser;
typedef struct Server aServer;
typedef struct SLink Link;
typedef struct ChanLink chanMember;
typedef struct SMode Mode;
typedef struct Watch aWatch;
typedef struct Ban aBan;
typedef struct Exempt anExempt;
typedef struct ListOptions LOpts;
typedef long ts_val;

typedef struct MotdItem aMotd;

typedef struct OMotdItem aOMotd;
typedef struct RulesItem aRules;

typedef struct t_crline aCRline;


#include "class.h"
#include "dbuf.h"		/* THIS REALLY SHOULDN'T BE HERE!!! --msa */

#define SM_MAXMODES 6


#define	HOSTLEN		84	/* Length of hostname.  Updated to */
				/* comply with RFC1123 */
				/* Changed from 63 to 84 in order to allow */
				/* for the hiddenhost prefixes - ShadwoMaster */

#define HOSTIPLEN	15	/* Length of dotted quad form of IP */

/* - Dianora  */


#define	NICKLEN		30

/* Necessary to put 9 here instead of 10  if  
 * s_msg.c/m_nick has been corrected.  This 
 * preserves compatibility with old * servers --msa 
 */

#define MAX_DATE_STRING 32	/* maximum string length for a date string */

#define	USERLEN		10
#define	REALLEN	 	50
#define	TOPICLEN	307
#define	CHANNELLEN	32
#define	PASSWDLEN 	63
#define	KEYLEN		23
#define	BUFSIZE		512	/* WARNING: *DONT* CHANGE THIS!!!! */
#define	MAXRECIPIENTS 	20
#define	MAXBANS	 	60
#define MAXEXEMPTS	60

#define MOTDLINELEN	90

#define        MAXSILES        10
#define        MAXSILELENGTH   128

#define MAXDCCALLOW 5
#define DCC_LINK_ME	0x01	/* This is my dcc allow */
#define DCC_LINK_REMOTE 0x02	/* I need to remove these dcc allows from these clients when I die */

#define	USERHOST_REPLYLEN	(NICKLEN+HOSTLEN+USERLEN+5)

/*
 * 'offsetof' is defined in ANSI-C. The following definition * is not
 * absolutely portable (I have been told), but so far * it has worked
 * on all machines I have needed it. The type * should be size_t but...
 * --msa
 */

#ifndef offsetof
#define	offsetof(t,m) (int)((&((t *)0L)->m))
#endif

#define	elementsof(x) (sizeof(x)/sizeof(x[0]))

/* flags for bootup options (command line flags) */

#define	BOOT_CONSOLE 1
#define	BOOT_QUICK	 2
#define	BOOT_DEBUG	 4
#define	BOOT_INETD	 8
#define	BOOT_TTY	   16
#define	BOOT_OPER	   32
#define	BOOT_AUTODIE 64
#define BOOT_STDERR	 128
#define	STAT_LOG	   -6	/* logfile for -x */
#define	STAT_MASTER	-5	/* Local ircd master before identification */
#define	STAT_CONNECTING	-4
#define	STAT_HANDSHAKE	-3
#define	STAT_ME		      -2
#define	STAT_UNKNOWN	-1
/* the line of truth lies here (truth == registeredness) */
#define	STAT_SERVER	0
#define	STAT_CLIENT	    1

/* status macros. */

#define	IsRegisteredUser(x)	((x)->status == STAT_CLIENT)
#define	IsRegistered(x)		((x)->status >= STAT_SERVER)
#define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
#define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
#define	IsMe(x)			((x)->status == STAT_ME)
#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN || (x)->status == STAT_MASTER)
#define	IsServer(x)		((x)->status == STAT_SERVER)
#define	IsClient(x)		((x)->status == STAT_CLIENT)
#define	IsLog(x)		((x)->status == STAT_LOG)

#define	SetMaster(x)		((x)->status = STAT_MASTER)
#define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
#define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
#define	SetMe(x)		((x)->status = STAT_ME)
#define	SetUnknown(x)		((x)->status = STAT_UNKNOWN)
#define	SetServer(x)		((x)->status = STAT_SERVER)
#define	SetClient(x)		((x)->status = STAT_CLIENT)
#define	SetLog(x)		((x)->status = STAT_LOG)

#define	FLAGS_PINGSENT     0x00000001	/* Unreplied ping sent */
#define	FLAGS_DEADSOCKET   0x00000002	/* Local socket is dead--Exiting soon */
#define	FLAGS_KILLED       0x00000004	/* Prevents "QUIT" from being sent for this */
#define	FLAGS_BLOCKED      0x00000008	/* socket is in a blocked condition */
#define FLAGS_REJECT_HOLD  0x00000010	/* client has been klined */
#define	FLAGS_CLOSING      0x00000020	/* set when closing to suppress errors */
#define	FLAGS_LISTEN       0x00000040	/* used to mark clients which we listen() on */
#define	FLAGS_CHKACCESS    0x00000080	/* ok to check clients access if set */
#define	FLAGS_DOINGDNS	   0x00000100	/* client is waiting for a DNS response */
#define	FLAGS_AUTH	   0x00000200	/* client is waiting on rfc931 response */
#define	FLAGS_WRAUTH	   0x00000400	/* set if we havent writen to ident server */
#define	FLAGS_LOCAL	   0x00000800	/* set for local clients */
#define	FLAGS_GOTID	   0x00001000	/* successful ident lookup achieved */
#define	FLAGS_DOID	   0x00002000	/* I-lines say must use ident return */
#define	FLAGS_NONL	   0x00004000	/* No \n in buffer */
#define FLAGS_NORMALEX     0x00008000	/* Client exited normally */
#define FLAGS_SENDQEX      0x00010000	/* Sendq exceeded */
#define FLAGS_IPHASH       0x00020000	/* iphashed this client */
#define FLAGS_ULINE 	   0x00040000	/* client is U-lined */
#define FLAGS_USERBURST	   0x00080000	/* server in nick/channel netburst */
#define FLAGS_TOPICBURST   0x00100000	/* server in topic netburst */
#define FLAGS_BURST	(FLAGS_USERBURST | FLAGS_TOPICBURST)
#define FLAGS_SOBSENT      0x00200000	/* we've sent an SOB, just have to send an EOB */
#define FLAGS_EOBRECV      0x00400000	/* we're waiting on an EOB */
#define FLAGS_MAP     	   0x00800000	/* Show this entry in /map */
#define FLAGS_BAD_DNS	   0x01000000	/* spoofer-guy */
#define FLAGS_SERV_NEGO	   0x02000000	/* This is a server that has passed connection tests,
					   but is a stat < 0 for handshake purposes */
#define FLAGS_RC4IN        0x04000000	/* This link is rc4 encrypted. */
#define FLAGS_RC4OUT       0x08000000	/* This link is rc4 encrypted. */
#define FLAGS_ZIPPED_IN	   0x10000000	/* This link is gzipped. */
#define FLAGS_ZIPPED_OUT   0x20000000	/* This link is gzipped. */

/* #define      SetUnixSock(x)          ((x)->flags |= FLAGS_UNIX) */

#define	IsListening(x)		((x)->flags & FLAGS_LISTEN)

#define	DoAccess(x)		((x)->flags & FLAGS_CHKACCESS)

#define	IsLocal(x)		((x)->flags & FLAGS_LOCAL)

#define	IsDead(x)		((x)->flags & FLAGS_DEADSOCKET)

#define	SetDNS(x)		((x)->flags |= FLAGS_DOINGDNS)
#define	DoingDNS(x)		((x)->flags & FLAGS_DOINGDNS)
#define	SetAccess(x)		((x)->flags |= FLAGS_CHKACCESS)
#define	DoingAuth(x)		((x)->flags & FLAGS_AUTH)
#define	NoNewLine(x)		((x)->flags & FLAGS_NONL)


#define	ClearDNS(x)		((x)->flags &= ~FLAGS_DOINGDNS)
#define	ClearAuth(x)		((x)->flags &= ~FLAGS_AUTH)
#define	ClearAccess(x)		((x)->flags &= ~FLAGS_CHKACCESS)

#define SetNegoServer(x)	((x)->flags |= FLAGS_SERV_NEGO)
#define IsNegoServer(x)		((x)->flags & FLAGS_SERV_NEGO)
#define ClearNegoServer(x)	((x)->flags &= ~FLAGS_SERV_NEGO)
#define IsRC4OUT(x)		((x)->flags & FLAGS_RC4OUT)
#define SetRC4OUT(x)		((x)->flags |= FLAGS_RC4OUT)
#define IsRC4IN(x)		((x)->flags & FLAGS_RC4IN)
#define SetRC4IN(x)		((x)->flags |= FLAGS_RC4IN)
#define RC4EncLink(x)		(((x)->flags & (FLAGS_RC4IN|FLAGS_RC4OUT)) == (FLAGS_RC4IN|FLAGS_RC4OUT))

#define ZipIn(x)		((x)->flags & FLAGS_ZIPPED_IN)
#define SetZipIn(x)		((x)->flags |= FLAGS_ZIPPED_IN)
#define ZipOut(x)		((x)->flags & FLAGS_ZIPPED_OUT)
#define SetZipOut(x)		((x)->flags |= FLAGS_ZIPPED_OUT)


#ifdef SOCKSPORT
#define FLAGS2_PROXY	   0x00000001	/* client is waiting on a rfc1928 response */
#define FLAGS2_WRPROXY     0x00000002	/* set if we havent written to socks/proxy server */
#define	FLAGS2_GOTPROXY    0x00000004	/* client has an open socks/proxy server */

#define	DoingProxy(x)		((x)->flags2 & FLAGS2_PROXY)
#define	ClearProxy(x)		((x)->flags2 &= ~FLAGS2_PROXY)
#endif

/* Capabilities of the ircd */

#define CAPAB_TS3     0x0000001	/* Supports the TS3 Protocal */
#define CAPAB_NOQUIT  0x0000002	/* Supports NOQUIT */
#define CAPAB_NSJOIN  0x0000004	/* server supports new smart sjoin */
#define CAPAB_BURST   0x0000008	/* server supports BURST command */
#define CAPAB_UNCONN  0x0000010	/* server supports UNCONNECT */
#define CAPAB_SSJ3    0x0000020	/* Server supports our new SSJOIN protocol */
#define CAPAB_DKEY    0x0000040	/* server supports dh-key exchange using "DKEY" */
#define CAPAB_ZIP     0x0000080	/* server supports gz'd links */
#define CAPAB_DOZIP   0x0000100	/* output to this link shall be gzipped */
#define CAPAB_DODKEY  0x0000200	/* do I do dkey with this link? */
#define CAPAB_NICKIP  0x0000400	/* IP in the NICK line? */
#define CAPAB_TS5 0x0000800	/* Supports the TS5 Protocol */
#define CAPAB_CLIENT 0x0001000 /* Supports CLIENT */


#define SetTS3(x)   	((x)->capabilities |= CAPAB_TS3)
#define IsTS3(x)       	((x)->capabilities & CAPAB_TS3)

#define SetNoQuit(x) 	((x)->capabilities |= CAPAB_NOQUIT)
#define IsNoQuit(x) 	((x)->capabilities & CAPAB_NOQUIT)

#define SetSSJoin(x)	((x)->capabilities |= CAPAB_NSJOIN)
#define IsSSJoin(x)	((x)->capabilities & CAPAB_NSJOIN)

#define SetBurst(x)	((x)->capabilities |= CAPAB_BURST)
#define IsBurst(x)	((x)->capabilities & CAPAB_BURST)

#define SetUnconnect(x)	((x)->capabilities |= CAPAB_UNCONN)
#define IsUnconnect(x)	((x)->capabilities & CAPAB_UNCONN)

#define SetSSJoin3(x)	((x)->capabilities |= CAPAB_SSJ3)
#define IsSSJoin3(x)	((x)->capabilities & CAPAB_SSJ3)

#define SetDKEY(x)	((x)->capabilities |= CAPAB_DKEY)
#define CanDoDKEY(x)    ((x)->capabilities & CAPAB_DKEY)
#define WantDKEY(x)	((x)->capabilities & CAPAB_DODKEY)	/* N: line, flag E */

#define SetZipCapable(x) ((x)->capabilities |= CAPAB_ZIP)
#define IsZipCapable(x)	((x)->capabilities & CAPAB_ZIP)
#define DoZipThis(x) 	((x)->capabilities & CAPAB_DOZIP)	/* this is set in N: line, flag Z */

#define SetNICKIP(x)    ((x)->capabilities |= CAPAB_NICKIP)
#define IsNICKIP(x)     ((x)->capabilities & CAPAB_NICKIP)

#define SetTS5(x)	((x)->capabilities |= CAPAB_TS5)
#define IsTS5(x)		((x)->capabilities & CAPAB_TS5)

#define SetClientCapable(x)	((x)->capabilities |= CAPAB_CLIENT)
#define IsClientCapable(x)	((x)->capabilities & CAPAB_CLIENT)

/* flag macros. */
#define IsULine(x) ((x)->flags & FLAGS_ULINE)


/* User Modes */
#define UMODE_o     	0x00000001	/* umode +o - Oper */
#define UMODE_O     	0x00000002	/* umode +O - Local Oper */
#define UMODE_i     	0x00000004	/* umode +i - Invisible */
#define UMODE_w     	0x00000008	/* umode +w - Get wallops */
#define UMODE_s     	0x00000010	/* umode +s - Server notices */
#define UMODE_c     	0x00000020	/* umode +c - Client connections/exits */
#define UMODE_r     	0x00000040	/* umode +r - registered nick */
#define UMODE_k     	0x00000080	/* umode +k - Server kill messages */
#define UMODE_f     	0x00000100	/* umode +f - Server flood messages */
#define UMODE_y		0x00000200	/* umode +y - Stats/links */
#define UMODE_D     	0x00000400	/* umode +D - pseudo/hidden, has seen dcc warning message */
#define UMODE_g     	0x00001000	/* umode +g - Globops */
#define UMODE_C     	0x00002000	/* umode +C - Chatops */
#define UMODE_a     	0x00004000	/* umode +a - Services Operator */
#define UMODE_A     	0x00008000	/* umode +A - Server Admin */
#define UMODE_n     	0x00010000	/* umode +n - Routing Notices */
#define UMODE_h     	0x00020000	/* umode +h - Helper */
#define UMODE_m     	0x00040000	/* umode +m - spambot notices */
#define UMODE_x     	0x00080000	/* umode +x - hidden hostname */
#define UMODE_T	    	0x00100000	/* umode +T - Network Admin */
#define UMODE_t     	0x00200000	/* umode +t - Network Co Admin */
#define UMODE_J		0x00400000	/* umode +J - Server Co Admin */
#define UMODE_j		0x00800000	/* umode +j - Guest Admin */
#define UMODE_Z		0x01000000	/* umode +Z - Services Root Admin */
#define UMODE_P		0x02000000	/* umode +P - Services Admin */
#define UMODE_S		0x04000000	/* umode +S - Services Client */
#define UMODE_p		0x08000000	/* umode +p - Protected Oper */
#define UMODE_F		0x10000000	/* umode +F - Global client connections/exits */
#define UMODE_d		0x20000000	/* umode +d - Debug Info */
#define UMODE_e		0x40000000	/* umode +e - See DCC send warnings */
#define UMODE_W		0x80000000	/* umode +W - Opers can see when a user /whois's them */
/*
* You cant fit anymore into umode.. youve been warned.
*/

/* for sendto_realops_lev */

#define CCONN_LEV	1
#define REJ_LEV		2
#define SKILL_LEV	3
#define SPY_LEV		4
#define DEBUG_LEV	5
#define FLOOD_LEV 	6
#define SPAM_LEV 	7
#define DCCSEND_LEV	8

#define	SEND_UMODES (UMODE_i|UMODE_o|UMODE_O|UMODE_w|UMODE_r|UMODE_a|UMODE_A|UMODE_h|UMODE_x|UMODE_T|UMODE_t|UMODE_J|UMODE_j|UMODE_Z|UMODE_P|UMODE_S|UMODE_p|UMODE_d|UMODE_D|UMODE_e|UMODE_W)
#define ALL_UMODES (SEND_UMODES|UMODE_s|UMODE_c|UMODE_r|UMODE_k|UMODE_f|UMODE_y|UMODE_D|UMODE_g|UMODE_C|UMODE_n|UMODE_h|UMODE_m|UMODE_O|UMODE_x|UMODE_T|UMODE_t|UMODE_J|UMODE_j|UMODE_Z|UMODE_P|UMODE_S|UMODE_p|UMODE_F|UMODE_d|UMODE_W)

#define LOCOP_UMODES (UMODE_O|UMODE_w|UMODE_s|UMODE_y|UMODE_D|UMODE_g|UMODE_n|UMODE_x|UMODE_h)
#define	FLAGS_ID (FLAGS_DOID|FLAGS_GOTID)

#define	IsLocOp(x)		((x)->umode & UMODE_O)
#define	SetLocOp(x)    		((x)->umode |= UMODE_O)
#define ClearLocOp(x)		((x)->umode &= ~UMODE_O)

#define	IsOper(x)		((x)->umode & UMODE_o)
#define	SetOper(x)		((x)->umode |= UMODE_o)
#define	ClearOper(x)		((x)->umode &= ~UMODE_o)

#define IsGuestAdmin(x)		((x)->umode & UMODE_j)
#define SetGuestAdmin(x)	((x)->umode |= UMODE_j)
#define ClearGuestAdmin(x)	((x)->umode &= ~UMODE_j)

#define IsServerCoAdmin(x)	((x)->umode & UMODE_J)
#define SetServerCoAdmin(x)	((x)->umode |= UMODE_J)
#define ClearServerCoAdmin(x)	((x)->umode &= ~UMODE_J)

#define IsServerAdmin(x)    	((x)->umode & UMODE_A)
#define SetServerAdmin(x)	((x)->umode |= UMODE_A)
#define ClearServerAdmin(x)   	((x)->umode &= ~UMODE_A)

#define IsNetCoAdmin(x)		((x)->umode & UMODE_t)
#define SetNetCoAdmin(x)	((x)->umode |= UMODE_t)
#define ClearNetCoAdmin(x)	((x)->umode &= ~UMODE_t)

#define IsNetAdmin(x)		((x)->umode & UMODE_T)
#define SetNetAdmin(x)		((x)->umode |= UMODE_T)
#define ClearNetAdmin(x)	((x)->umode &= ~UMODE_T)

#define IsServicesOper(x)   	((x)->umode & UMODE_a)
#define SetServicesOper(x)  	((x)->umode |= UMODE_a)
#define ClearServicesOper(x)  	((x)->umode &= ~UMODE_a)

#define IsServicesAdmin(x)	((x)->umode & UMODE_P)
#define SetServicesAdmin(x)	((x)->umode |= UMODE_P)
#define ClearServicesAdmin(x)	((x)->umode &= ~UMODE_P)

#define IsServicesRoot(x)	((x)->umode & UMODE_Z)
#define SetServicesRoot(x)	((x)->umode |= UMODE_Z)
#define ClearServicesRoot(x)	((x)->umode &= ~UMODE_Z)

#define	IsInvisible(x)		((x)->umode & UMODE_i)
#define	SetInvisible(x)		((x)->umode |= UMODE_i)
#define	ClearInvisible(x)	((x)->umode &= ~UMODE_i)

#define IsARegNick(x)   	((x)->umode & (UMODE_r))

#define IsRegNick(x)    	((x)->umode & UMODE_r)
#define SetRegNick(x)   	((x)->umode |= UMODE_r)

#define IsUmodef(x)   		((x)->umode & UMODE_f)

#define IsUmodec(x)   		((x)->umode & UMODE_c)
#define SendCConnNotice(x)	((x)->umode & UMODE_c)
#define SendRejNotice(x)	((x)->umode & UMODE_c)
#define ClearUmodec(x)  	((x)->umode &= ~UMODE_c)

#define IsUmodey(x)   		((x)->umode & UMODE_y)

#define IsUmodeD(x)   		((x)->umode & UMODE_D)
#define SeenDCCNotice(x)      	((x)->umode & UMODE_D)
#define SetDCCNotice(x)   	((x)->umode |= UMODE_D)

#define IsUmodeC(x)   		((x)->umode & UMODE_C)
#define SendChatops(x) 		((x)->umode & UMODE_C)
#define SetChatops(x)		((x)->umode |= UMODE_C)
#define ClearUmodeC(x)  	((x)->umode &= ~UMODE_C)

#define IsUmoden(x)   		((x)->umode & UMODE_n)
#define SendNetInfo(x)		((x)->umode & UMODE_n)
#define ClearUmoden(x)  	((x)->umode &= ~UMODE_n)

#define IsUmodem(x)   		((x)->umode & UMODE_m)

#define IsUmodeh(x)   		((x)->umode & UMODE_h)
#define IsHelpOp(x)		((x)->umode & UMODE_h)
#define SetHelpOp(x)		((x)->umode |= UMODE_h)
#define ClearHelpOp(x)		((x)->umode &= UMODE_h)
#define ClearUmodeh(x)  	((x)->umode &= ~UMODE_h)

#define IsUmodee(x)   		((x)->umode & UMODE_e)

#define	SendWallops(x)		((x)->umode & UMODE_w)
#define	SetWallops(x)  		((x)->umode |= UMODE_w)
#define	ClearWallops(x)		((x)->umode &= ~UMODE_w)

#define	SendServNotice(x)	((x)->umode & UMODE_s)

#define SendSkillNotice(x)	((x)->umode & UMODE_k)

#define SendSpyNotice(x)	((x)->umode & UMODE_y)
#define ClearUmodey(x)  	((x)->umode &= ~UMODE_y)

#define SendFloodNotice(x)  	((x)->umode & UMODE_f)
#define ClearUmodef(x)  	((x)->umode &= ~UMODE_f)

#define SendSpamNotice(x)  	((x)->umode & UMODE_m)
#define ClearUmodem(x)  	((x)->umode &= ~UMODE_m)

#define SendDebugNotice(x)	((x)->umode & UMODE_d)
#define IsUmoded(x)   		((x)->umode & UMODE_d)
#define ClearUmoded(x)  	((x)->umode &= ~UMODE_d)

#define SendGlobops(x) 		((x)->umode & UMODE_g)

#define SendRnotice(x) 		((x)->umode & UMODE_n)

#define IsHidden(x)             ((x)->umode & UMODE_x)
#define SetHidden(x)            ((x)->umode |= UMODE_x)
#define ClearHidden(x)		((x)->umode &= ~UMODE_x)

#define IsServicesClient(x)	((x)->umode & UMODE_S)
#define SetServicesClient(x)	((x)->umode |= UMODE_S)
#define ClearServicesClient(x)	((x)->umode &= ~UMODE_S)

#define IsProtectedOper(x)	((x)->umode & UMODE_p)
#define SetProtectedOper(x)	((x)->umode |= UMODE_p)
#define ClearProtectedOper(x)	((x)->umode &= ~UMODE_p)

#define SendGlobalConnect(x)	((x)->umode & UMODE_F)
#define SetGlobalConnect(x)	((x)->umode |= UMODE_F)
#define ClearGlobalConnect(x)	((x)->umode &= ~UMODE_F)

#define WantsWhois(x)		((x)->umode & UMODE_W)
#define SetWantsWhois(x)	((x)->umode |= UMODE_W)
#define ClearWantsWhois(x)	((x)->umode &= ~UMODE_W)

#define IsUmodee(x)   		((x)->umode & UMODE_e)
#define SendDCCNotice(x)	((x)->umode & UMODE_e)
#define ClearUmodee(x)  ((x)->umode &= ~UMODE_e)

#define	IsAnOper(x)		((x)->umode & (UMODE_o|UMODE_O))
#define IsSkoAdmin(x) 		(IsServerCoAdmin(x) || IsServerAdmin(x) || IsNetCoAdmin(x) || IsNetAdmin(x))
#define IsSkoServicesStaff(x)	(IsServicesOper(x) || IsServicesAdmin(x) || IsServicesRoot(x))
#define IsSkoNetAdmin(x)	(IsNetAdmin(x) || IsNetCoAdmin(x))

#define	IsPerson(x)		((x)->user && IsClient(x))

#define	IsPrivileged(x)		(IsAnOper(x) || IsServer(x))

/* flags2 macros. */

#define IsRestricted(x)		((x)->flags & FLAGS_RESTRICTED)
#define SetRestricted(x)	((x)->flags |= FLAGS_RESTRICTED)
/* Oper flags */

/* defined operator access levels */
#define OFLAG_REHASH		0x00000001	/* Oper can /rehash server */
#define OFLAG_DIE		0x00000002	/* Oper can /die the server */
#define OFLAG_RESTART		0x00000004	/* Oper can /restart the server */
#define OFLAG_HELPOP		0x00000010	/* Oper can send /HelpOps */
#define OFLAG_GLOBOP		0x00000020	/* Oper can send /GlobOps */
#define OFLAG_WALLOP		0x00000040	/* Oper can send /WallOps */
#define OFLAG_LOCOP		0x00000080	/* Oper can send /LocOps */
#define OFLAG_LROUTE		0x00000100	/* Oper can do local routing */
#define OFLAG_GROUTE		0x00000200	/* Oper can do global routing */
#define OFLAG_LKILL		0x00000400	/* Oper can do local kills */
#define OFLAG_GKILL		0x00000800	/* Oper can do global kills */
#define OFLAG_KLINE		0x00001000	/* Oper can /kline users */
#define OFLAG_UNKLINE		0x00002000	/* Oper can /unkline users */
#define OFLAG_LNOTICE		0x00004000	/* Oper can send local serv notices */
#define OFLAG_GNOTICE		0x00008000	/* Oper can send global notices */
#define OFLAG_SERVADMIN		0x00010000	/* Server Admin */
#define OFLAG_UMODEC		0x00020000	/* Oper can set umode +c */
#define OFLAG_UMODEF		0x00040000	/* Oper can set umode +f */
#define OFLAG_SERVICESOPER	0x00080000	/* Oper can be a services operator */
#define OFLAG_ZLINE		0x00100000	/* Oper can use /zline and /unzline */
#define OFLAG_UMODEY    	0x00200000	/* Oper can set umode +y */
#define OFLAG_UMODED    	0x00400000	/* Oper can set umode +d */
#define OFLAG_CHATOPS    	0x00800000	/* Oper can set umode +C */
#define OFLAG_NETADMIN		0x01000000	/* Network Admin */
#define OFLAG_NETCOADMIN	0x02000000	/* Network Co Admin */
#define OFLAG_SERVCOADMIN	0x04000000	/* Server Co Admin */
#define OFLAG_GUESTADMIN	0x08000000	/* Guest Admin */
#define OFLAG_SERVICESADMIN	0x10000000	/* Services Admin */
#define OFLAG_SERVICESROOT	0x20000000	/* Services Root */
#define OFLAG_PROTECTEDOPER	0x40000000	/* Protectec Operator */
#define OFLAG_UMODEE		0x80000000	/* Oper can set umode +e */


#define OFLAG_LOCAL	(OFLAG_REHASH|OFLAG_HELPOP|OFLAG_GLOBOP|OFLAG_WALLOP|OFLAG_LOCOP|OFLAG_LROUTE|OFLAG_LKILL|OFLAG_KLINE|OFLAG_UNKLINE|OFLAG_LNOTICE|OFLAG_UMODEC|OFLAG_UMODEF|OFLAG_UMODEE)
#define OFLAG_GLOBAL	(OFLAG_REHASH|OFLAG_HELPOP|OFLAG_GLOBOP|OFLAG_WALLOP|OFLAG_LOCOP|OFLAG_LROUTE|OFLAG_LKILL|OFLAG_KLINE|OFLAG_UNKLINE|OFLAG_LNOTICE|OFLAG_UMODEC|OFLAG_UMODEF|OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE|OFLAG_UMODEE)
#define OFLAG_ISGLOBAL	(OFLAG_GROUTE|OFLAG_GKILL|OFLAG_GNOTICE)

#define OPIsGuestAdmin(x)	((x)->oflag & OFLAG_GUESTADMIN)
#define OPSetGuestAdmin(x)	((x)->oflag |= OFLAG_GUESTADMIN)
#define OPClearGuestAdmin(x)	((x)->oflag &= ~OFLAG_GUESTADMIN)

#define OPIsServerCoAdmin(x)	((x)->oflag & OFLAG_SERVCOADMIN)
#define OPSetServerCoAdmin(x)	((x)->oflag |= OFLAG_SERVCOADMIN)
#define OPClearServerCoAdmin(x)	((x)->oflag &= ~OFLAG_SERVCOADMIN)

#define OPIsServerAdmin(x)	((x)->oflag & OFLAG_SERVADMIN)
#define OPSetServerAdmin(x)	((x)->oflag |= OFLAG_SERVADMIN)
#define OPClearServerAdmin(x)	((x)->oflag &= ~OFLAG_SERVADMIN)

#define OPIsNetCoAdmin(x)	((x)->oflag & OFLAG_NETCOADMIN)
#define OPSetNetCoAdmin(x)	((x)->oflag |= OFLAG_NETCOADMIN)
#define OPClearNetCoAdmin(x)	((x)->oflag &= ~OFLAG_NETCOADMIN)

#define OPIsNetAdmin(x)		((x)->oflag & OFLAG_NETADMIN)
#define OPSetNetAdmin(x)	((x)->oflag |= OFLAG_NETADMIN)
#define OPClearNetAdmin(x)	((x)->oflag &= ~OFLAG_NETADMIN)

#define OPIsServicesOper(x)	((x)->oflag & OFLAG_SERVICESOPER)
#define OPSetServicesOper(x) 	((x)->oflag |= OFLAG_SERVICESOPER)
#define OPClearServicesOper(x)	((x)->oflag &= ~OFLAG_SERVICESOPER)

#define OPIsServicesAdmin(x)	((x)->oflag & OFLAG_SERVICESADMIN)
#define OPSetServicesAdmin(x)	((x)->oflag |= OFLAG_SERVICESADMIN)
#define OPClearServicesAdmin(x)	((x)->oflag &= ~ OFLAG_SERVICESADMIN)

#define OPIsServicesRoot(x)	((x)->oflag & OFLAG_SERVICESROOT)
#define OPSetServicesRoot(x)	((x)->oflag |= OFLAG_SERVICESROOT)
#define OPClearServicesRoot(x)	((x)->oflag &= ~OFLAG_SERVICESROOT)

#define OPCanZline(x)		((x)->oflag & OFLAG_ZLINE)
#define OPSetZLine(x)		((x)->oflag |= OFLAG_ZLINE)
#define OPClearZLine(x)		((x)->oflag &= ~OFLAG_ZLINE)

#define OPCanRehash(x)		((x)->oflag & OFLAG_REHASH)
#define OPSetRehash(x)		((x)->oflag |= OFLAG_REHASH)
#define OPClearRehash(x)	((x)->oflag &= ~OFLAG_REHASH)

#define OPCanDie(x)		((x)->oflag & OFLAG_DIE)
#define OPSetDie(x)		((x)->oflag |= OFLAG_DIE)
#define OPClearDie(x)		((x)->oflag &= ~OFLAG_DIE)

#define OPCanRestart(x)		((x)->oflag & OFLAG_RESTART)
#define OPSetRestart(x)		((x)->oflag |= OFLAG_RESTART)
#define OPClearRestart(x)	((x)->oflag &= ~OFLAG_RESTART)

#define OPCanHelpOp(x)		((x)->oflag & OFLAG_HELPOP)
#define OPSetHelpOp(x)		((x)->oflag |= OFLAG_HELPOP)
#define OPClearHelpOp(x)	((x)->oflag &= ~OFLAG_HELPOP)

#define OPCanGlobOps(x)		((x)->oflag & OFLAG_GLOBOP)
#define OPSetGlobOps(x)		((x)->oflag |= OFLAG_GLOBOP)
#define OPClearGlobOps(x)	((x)->oflag &= ~OFLAG_GLOBOP)

#define OPCanWallOps(x)		((x)->oflag & OFLAG_WALLOP)
#define OPSetWallOps(x)		((x)->oflag |= OFLAG_WALLOP)
#define OPClearWallOps(x)	((x)->oflag &= ~OFLAG_WALLOP)

#define OPCanLocOps(x)		((x)->oflag & OFLAG_LOCOP)
#define OPSetLocOps(x)		((x)->oflag |= OFLAG_LOCOP)
#define OPClearLocOps(x)	((x)->oflag &= ~OFLAG_LOCOP)

#define OPCanLRoute(x)		((x)->oflag & OFLAG_LROUTE)
#define OPSetLRoute(x)		((x)->oflag |= OFLAG_LROUTE)
#define OPClearLRoute(x)	((x)->oflag &= ~OFLAG_LROUTE)

#define OPCanGRoute(x)		((x)->oflag & OFLAG_GROUTE)
#define OPSetGRoute(x)		((x)->oflag |= OFLAG_GROUTE)
#define OPClearGRoute(x)	((x)->oflag &= ~OFLAG_GROUTE)

#define OPCanLKill(x)		((x)->oflag & OFLAG_LKILL)
#define OPSetLKill(x)		((x)->oflag |= OFLAG_LKILL)
#define OPClearLKill(x)		((x)->oflag &= ~OFLAG_LKILL)

#define OPCanGKill(x)		((x)->oflag & OFLAG_GKILL)
#define OPSetGKill(x)		((x)->oflag |= OFLAG_GKILL)
#define OPClearGKill(x)		((x)->oflag &= ~OFLAG_GKILL)

#define OPCanKline(x)		((x)->oflag & OFLAG_KLINE)
#define OPSetKline(x)		((x)->oflag |= OFLAG_KLINE)
#define OPClearKline(x)		((x)->oflag &= ~OFLAG_KLINE)

#define OPCanUnKline(x)		((x)->oflag & OFLAG_UNKLINE)
#define OPSetUnKline(x)		((x)->oflag |= OFLAG_UNKLINE)
#define OPClearUnKline(x)	((x)->oflag &= ~OFLAG_UNKLINE)

#define OPCanLNotice(x)		((x)->oflag & OFLAG_LNOTICE)
#define OPSetLNotice(x)		((x)->oflag |= OFLAG_LNOTICE)
#define OPClearLNotice(x)	((x)->oflag &= ~OFLAG_LNOTICE)

#define OPCanGNotice(x)		((x)->oflag & OFLAG_GNOTICE)
#define OPSetGNotice(x)		((x)->oflag |= OFLAG_GNOTICE)
#define OPClearGNotice(x)	((x)->oflag &= ~OFLAG_GNOTICE)

#define OPCanUModeC(x)		((x)->oflag & OFLAG_UMODEC)
#define OPSetUModeC(x)		((x)->oflag |= OFLAG_UMODEC)
#define OPClearUModeC(x)	((x)->oflag &= ~OFLAG_UMODEC)

#define OPCanUModeF(x)		((x)->oflag & OFLAG_UMODEF)
#define OPSetUModeF(x)		((x)->oflag |= OFLAG_UMODEF)
#define OPClearUModeF(x)	((x)->oflag &= ~OFLAG_UMODEF)

#define OPCanUModeY(x)  	((x)->oflag & OFLAG_UMODEY)
#define OPSetUModeY(x)  	((x)->oflag |= OFLAG_UMODEY)
#define OPClearUModeY(x)        ((x)->oflag &= ~OFLAG_UMODEY)

#define OPCanUModeD(x)  	((x)->oflag & OFLAG_UMODED)
#define OPSetUModeD(x)  	((x)->oflag |= OFLAG_UMODED)
#define OPClearUModeD(x)        ((x)->oflag &= ~OFLAG_UMODED)

#define OPCanUModeE(x)		((x)->oflag & OFLAG_UMODEE)
#define OPSetUModeE(x)		((x)->oflag |= OFLAG_UMODEE)
#define OPClearUModeE(x)	((x)->oflag &= ~OFLAG_UMODEE)

#define OPCanChatops(x)		((x)->oflag & OFLAG_CHATOPS)
#define OPSetChatops(x)		((x)->oflag |= OFLAG_CHATOPS)
#define OPClearChatops(x)	((x)->oflag &= ~OFLAG_CHATOPS)

#define OPIsProtected(x)	((x)->oflag & OFLAG_PROTECTEDOPER)
#define OPSetProtected(x)	((x)->oflag |= OFLAG_PROTECTEDOPER)
#define OPClearProtected(x)	((x)->oflag &= ~OFLAG_PROTECTECOPER)

/* defined debugging levels */
#define	DEBUG_FATAL  0
#define	DEBUG_ERROR  1		/* report_error() and other errors that are found */
#define	DEBUG_NOTICE 3
#define	DEBUG_DNS    4		/* used by all DNS related routines - a *lot* */
#define	DEBUG_INFO   5		/* general usful info */
#define	DEBUG_NUM    6		/* numerics */
#define	DEBUG_SEND   7		/* everything that is sent out */
#define	DEBUG_DEBUG  8		/* anything to do with debugging, ie unimportant :) */
#define	DEBUG_MALLOC 9		/* malloc/free calls */
#define	DEBUG_LIST  10		/* debug list use */
/* defines for curses in client */
#define	DUMMY_TERM	0
#define	CURSES_TERM	1
#define	TERMCAP_TERM	2

struct Counter
{
  int server;			/* servers */
  int myserver;			/* my servers */
  int myulined;			/* my ulined servers */
  int oper;			/* Opers */
  int chan;			/* Channels */
  int local;			/* Local Clients */
  int total;			/* total clients */
  int invisi;			/* invisible clients */
  int unknown;			/* unknown connections */
  int max_loc;			/* MAX local clients */
  int max_tot;			/* MAX global clients */
  ts_val start;			/* when we started collecting info */
  u_long today;			/* Client Connections today */
  ts_val day;			/* when today started */
  u_long weekly;		/* connections this week */
  ts_val week;			/* when this week started */
  u_long monthly;		/* connections this month */
  ts_val month;			/* when this month started */
  u_long yearly;		/* this is gonna be big */
  ts_val year;			/* when this year started (HEH!) */
};

struct MotdItem
{
  char line[MOTDLINELEN];
  struct MotdItem *next;
};
struct OMotdItem
{
  char line[MOTDLINELEN];
  struct OMotdItem *next;
};
struct RulesItem
{
  char line[MOTDLINELEN];
  struct RulesItem *next;
};



/*
 * lets speed this up... also removed away information. *tough* Dianora
 */
typedef struct Whowas
{
  int hashv;
  char name[NICKLEN + 1];
  char username[USERLEN + 1];
  char hostname[HOSTLEN + 1];
  char virthostname[HOSTLEN + 1];
  char hostipname[HOSTLEN + 1];
  int washidden;
  char *servername;
  char realname[REALLEN + 1];
  time_t logoff;
  struct Client *online;	/* Pointer to new nickname for chasing or NULL */

  struct Whowas *next;		/* for hash table... */

  struct Whowas *prev;		/* for hash table... */
  struct Whowas *cnext;		/* for client struct linked list */
  struct Whowas *cprev;		/* for client struct linked list */
}
aWhowas;

struct ConfItem
{
  unsigned int status;		/* If CONF_ILLEGAL, delete when no clients */
  unsigned int flags;		/* i-lines and akills use this */
  int clients;			/* Number of *LOCAL* clients using this */
  struct in_addr ipnum;		/* ip number of host field */
  char *host;
  char *passwd;
  char *name;
  int port;
  time_t hold;			/* Hold action until this time (calendar time) */
  aClass *class;		/* Class of connection */
  struct ConfItem *next;
};

#define	CONF_ILLEGAL	0x80000000
#define	CONF_MATCH		0x40000000

/* #define      CONF_QUARANTINED_SERVER 0x0001 */
#define	CONF_CLIENT		0x0002
#define	CONF_CONNECT_SERVER	0x0004
#define	CONF_NOCONNECT_SERVER	0x0008
#define	CONF_LOCOP		0x0010
#define	CONF_OPERATOR		0x0020
#define	CONF_ME			0x0040
#define	CONF_KILL		0x0080
#define	CONF_ADMIN		0x0100
#define	CONF_CLASS		0x0200
#define	CONF_SERVICE		0x0400
#define	CONF_LEAF		0x0800
#define	CONF_LISTEN_PORT	0x1000
#define	CONF_HUB		0x2000
#define CONF_ELINE		0x4000
#define CONF_FLINE		0x8000
#define	CONF_ZLINE		0x20000
#define CONF_QUARANTINED_NICK 	0x40000
#define CONF_ULINE 		0x80000
#define CONF_DRPASS		0x100000	/* die/restart pass, from df465 */
#define CONF_AKILL		0x200000
#define CONF_SQLINE     	0x400000
#define CONF_GCOS               0x800000
#define CONF_SGLINE             0x1000000
#define CONF_SZLINE             0x2000000	/* Services placed zlines */
#define CONF_VHOST		0x4000000
#define CONF_QUARANTINED_CHAN	0x8000000
#define CONF_QUARANTINE         (CONF_QUARANTINED_NICK|CONF_QUARANTINED_CHAN)
#define	CONF_OPS		(CONF_OPERATOR | CONF_LOCOP)
#define	CONF_SERVER_MASK	(CONF_CONNECT_SERVER | CONF_NOCONNECT_SERVER)
#define	CONF_CLIENT_MASK	(CONF_CLIENT | CONF_SERVICE | CONF_OPS | CONF_SERVER_MASK)
#define	IsIllegal(x)	((x)->status & CONF_ILLEGAL)
#ifdef LITTLE_I_LINES
#define CONF_FLAGS_LITTLE_I_LINE	0x0001
#endif

/* Client structures */
struct User
{
  Link *channel;		/* chain of channel pointer blocks */
  Link *invited;		/* chain of invite pointer blocks */
  char *away;			/* pointer to away message */
  time_t last;
  int joined;			/* number of channels joined */
  char username[USERLEN + 1];
  char host[HOSTLEN + 1];
  char virthost[HOSTLEN + 1];
  char *server;			/* pointer to scached server name */
#ifdef OS_SOLARIS
  uint_t servicestamp;		/* solaris is gay -epi */
#else
  u_int32_t servicestamp;	/* Services id - Raistlin */
#endif
  /*
   * In a perfect world the 'server' name should not be needed, a
   * pointer to the client describing the server is enough. 
   * Unfortunately, in reality, server may not yet be in links while
   * USER is introduced... --msa
   */
  Link *silence;		/* chain of silenced users */
  LOpts *lopt;			/* Saved /list options */
  Link *dccallow;		/* chain of dcc send allowed users */
};

struct Server
{
  char *up;			/* Pointer to scache name */
  char bynick[NICKLEN + 1];
  char byuser[USERLEN + 1];
  char byhost[HOSTLEN + 1];
  aConfItem *nline;		/* N-line pointer for this server */
  int dkey_flags;		/* dkey flags */
#ifdef HAVE_ENCRYPTION_ON
  void *sessioninfo_in;		/* pointer to opaque sessioninfo structure */
  void *sessioninfo_out;	/* pointer to opaque sessioninfo structure */
  void *rc4_in;			/* etc */
  void *rc4_out;		/* etc */
#endif
  void *zip_out;
  void *zip_in;
};

struct Client
{
  struct Client *next, *prev, *hnext;
  anUser *user;			/* ...defined, if this is a User */
  aServer *serv;		/* ...defined, if this is a server */
  aWhowas *whowas;		/* Pointers to whowas structs */
  time_t lasttime;		/* ...should be only LOCAL clients? --msa */
  time_t firsttime;		/* time client was created */
  time_t since;			/* last time we parsed something */
  ts_val tsinfo;		/* TS on the nick, SVINFO on servers */
  long flags;			/* client flags */
  long flags2;			/* More client flags. Ran out of space - ShadowMaster */
  long umode;			/* We can illeviate overflow this way */
  aClient *from;		/* == self, if Local Client, *NEVER* NULL! */
  aClient *uplink;		/* this client's uplink to the network */
  int fd;			/* >= 0, for local clients */
  int hopcount;			/* number of servers to this 0 = local */
  short status;			/* Client type */
  char nicksent;
  char name[HOSTLEN + 1];	/* Unique name of the client, nick or host */
  char info[REALLEN + 1];	/* Free form additional client information */
#ifdef FLUD
  Link *fludees;
#endif

  struct in_addr ip;		/* keep real ip# too */
  char hostip[HOSTIPLEN + 1];	/* Keep real ip as string too - Dianora */

  Link *watch;			/* user's watch list */
  int watches;			/* how many watches this user has set */


  /*
   * The following fields are allocated only for local clients 
   * (directly connected to this server with a socket.  The first
   * of them MUST be the "count"--it is the field to which the
   * allocation is tied to! Never refer to  these fields, if (from != self).
   */

  int count;			/* Amount of data in buffer */
#ifdef FLUD
  time_t fludblock;
  struct fludbot *fluders;
#endif
#ifdef ANTI_SPAMBOT
  time_t last_join_time;	/* when this client last joined a channel */
  time_t last_leave_time;	/* when this client last left a channel */
  int join_leave_count;		/*
				 * count of JOIN/LEAVE in less 
				 * than MIN_JOIN_LEAVE_TIME seconds */
  int oper_warn_count_down;	/*
				 * warn opers of this possible spambot 
				 * every time this gets to 0 */
#endif
  char buffer[BUFSIZE];		/* Incoming message buffer */
  short lastsq;			/* # of 2k blocks when sendqueued called last */
  struct DBuf sendQ;		/* Outgoing message queue--if socket full */
  struct DBuf recvQ;		/* Hold for data incoming yet to be parsed */
  long sendM;			/* Statistics: protocol messages send */
  long sendK;			/* Statistics: total k-bytes send */
  long receiveM;		/* Statistics: protocol messages received */
  long receiveK;		/* Statistics: total k-bytes received */
  u_short sendB;		/* counters to count upto 1-k lots of bytes */
  u_short receiveB;		/* sent and received. */
  long lastrecvM;		/* to check for activity --Mika */
  int priority;
  aClient *acpt;		/* listening client which we accepted from */
  Link *confs;			/* Configuration record associated */
  int authfd;			/* fd for rfc931 authentication */
#ifdef SOCKSPORT
  int proxyfd;			/* fd for rfc1928 checking */
#endif
  char username[USERLEN + 1];	/* username here now for auth stuff */
  unsigned short port;		/* and the remote port# too :-) */
  struct hostent *hostp;
#ifdef ANTI_NICK_FLOOD
  time_t last_nick_change;
  int number_of_nick_changes;
#endif
#ifdef NO_AWAY_FLUD
  time_t alas;			/* last time of away set */
  int acount;			/* count of away settings */
#endif

  char sockhost[HOSTLEN + 1];	/*
				 * This is the host name from
				 * the socket and after which the
				 * connection was accepted. */
  char passwd[PASSWDLEN + 1];
  /* try moving this down here to prevent weird problems... ? */
  int oflag;			/* Operator Flags */
  int sockerr;			/* what was the last error returned for this socket? */
  int capabilities;		/* what this server/client supports */
  int pingval;			/* cache client class ping value here */
  int sendqlen;			/* cache client max sendq here */
};


#define	CLIENT_LOCAL_SIZE sizeof(aClient)
#define	CLIENT_REMOTE_SIZE offsetof(aClient,count)
/* statistics structures */
struct stats
{
  unsigned int is_cl;		/* number of client connections */
  unsigned int is_sv;		/* number of server connections */
  unsigned int is_ni;		/* connection but no idea who it was */
  unsigned short is_cbs;	/* bytes sent to clients */
  unsigned short is_cbr;	/* bytes received to clients */
  unsigned short is_sbs;	/* bytes sent to servers */
  unsigned short is_sbr;	/* bytes received to servers */
  unsigned long is_cks;		/* k-bytes sent to clients */
  unsigned long is_ckr;		/* k-bytes received to clients */
  unsigned long is_sks;		/* k-bytes sent to servers */
  unsigned long is_skr;		/* k-bytes received to servers */
  time_t is_cti;		/* time spent connected by clients */
  time_t is_sti;		/* time spent connected by servers */
  unsigned int is_ac;		/* connections accepted */
  unsigned int is_ref;		/* accepts refused */
  unsigned int is_unco;		/* unknown commands */
  unsigned int is_wrdi;		/* command going in wrong direction */
  unsigned int is_unpf;		/* unknown prefix */
  unsigned int is_empt;		/* empty message */
  unsigned int is_num;		/* numeric message */
  unsigned int is_kill;		/* number of kills generated on collisions */
  unsigned int is_fake;		/* MODE 'fakes' */
  unsigned int is_asuc;		/* successful auth requests */
  unsigned int is_abad;		/* bad auth requests */
  unsigned int is_psuc;		/* successful proxy requests */
  unsigned int is_pbad;		/* bad proxy requests */
  unsigned int is_udp;		/* packets recv'd on udp port */
  unsigned int is_loc;		/* local connections made */
#ifdef FLUD
  unsigned int is_flud;		/* users/channels flood protected */
#endif				/* FLUD */
  char pad[16];			/* Make this an even 1024 bytes */
};

/* mode structure for channels */

struct SMode
{
  unsigned int mode;
  int limit;
  char key[KEYLEN + 1];
};

/* Message table structure */

struct Message
{
  char *cmd;
  int (*func) ();
  unsigned int count;		/* number of times command used */
  int parameters;
  char flags;

  /* bit 0 set means that this command is allowed to be used only on
   * the average of once per 2 seconds -SRB */

  /* I could have defined other bit maps to above instead of the next
     * two flags that I added. so sue me. -Dianora */

  char allow_unregistered_use;	/* flag if this command can be used 
				   * if unregistered */

  char reset_idle;		/* flag if this command causes idle time to be 
				   * reset */
  unsigned long bytes;
};

typedef struct msg_tree
{
  char *final;
  struct Message *msg;
  struct msg_tree *pointers[26];
}
MESSAGE_TREE;

/*
 * Move BAN_INFO information out of the SLink struct its _only_ used
 * for bans, no use wasting the memory for it in any other type of
 * link. Keep in mind, doing this that it makes it slower as more
 * Malloc's/Free's have to be done, on the plus side bans are a smaller
 * percentage of SLink usage. Over all, the th+hybrid coding team came
 * to the conclusion it was worth the effort.
 * 
 * - Dianora
 */

struct Ban
{
  char *banstr;
  char *who;
  time_t when;
  u_char type;
  aBan *next;
};

struct Exempt
{
  char *exemptstr;
  char *who;
  time_t when;
  u_char type;
  anExempt *next;
};

/* channel member link structure, used for chanmember chains */
struct ChanLink
{
  struct ChanLink *next;
  aClient *cptr;
  int flags;
  int bans;			/* for bquiet: number of bans against this user */
};

/* general link structure used for chains */

struct SLink
{
  struct SLink *next;
  union
  {
    aClient *cptr;
    aChannel *chptr;
    aConfItem *aconf;
    aBan *banptr;
    anExempt *exemptptr;
    aWatch *wptr;
    char *cp;
  }
  value;
  int flags;
};

/* channel structure */

struct Channel
{
  struct Channel *nextch, *prevch, *hnextch;
  int hashv;			/* raw hash value */
  Mode mode;
  char topic[TOPICLEN + 1];
  char topic_nick[NICKLEN + 1];
  time_t topic_time;
  int users;
  chanMember *members;
  Link *invites;
  aBan *banlist;
  anExempt *exemptlist;
  ts_val channelts;
#ifdef FLUD
  time_t fludblock;
  struct fludbot *fluders;
#endif
  char chname[CHANNELLEN + 1];
};
#ifdef TS5
#define	TS_CURRENT	5	/* current TS protocol version */
#else
#define	TS_CURRENT	3	/* current TS protocol version */
#endif
#define	TS_MIN		3	/* minimum supported TS protocol version */
#define	TS_DOESTS	0x20000000
#define	DoesTS(x)	((x)->tsinfo == TS_DOESTS)
/* Channel Related macros follow */

/* Channel related flags */

#define	CHFL_CHANOP     	0x0001	/* Channel operator */
#define	CHFL_VOICE      	0x0002	/* the power to speak */
#define	CHFL_DEOPPED 		0x0004	/* deopped by us, modes need to be bounced */
#define CHFL_HALFOP		0x0008	/* Channel Half Operator */
#define CHFL_DEHALFOPPED	0x0010	/* De Halfopped by us, mode needs to be bounced */
#define CHFL_CHANADMIN		0x0020	/* Channel Administrator */
#define CHFL_DEADMINED		0x0040	/* De Admined by us, mode needs to be bounced */
#define	CHFL_BAN		0x0080	/* ban channel flag */
#define CHFL_EXEMPT		0x0100	/* exempt ban channel flag */
/* ban mask types */

#define MTYP_FULL      0x01	/* mask is nick!user@host */
#define MTYP_USERHOST  0x02	/* mask is user@host */
#define MTYP_HOST      0x04	/* mask is host only */

/* Channel Visibility macros */

#define	MODE_CHANOP	    	CHFL_CHANOP
#define	MODE_VOICE	    	CHFL_VOICE
#define	MODE_DEOPPED  		CHFL_DEOPPED
#define MODE_HALFOP		CHFL_HALFOP
#define MODE_DEHALFOPPED	CHFL_DEHALFOPPED
#define MODE_CHANADMIN		CHFL_CHANADMIN
#define MODE_DEADMINED		CHFL_DEADMINED
#define	MODE_PRIVATE  		0x00000080
#define	MODE_SECRET   		0x00000100
#define	MODE_MODERATED  	0x00000200
#define	MODE_TOPICLIMIT 	0x00000400
#define	MODE_INVITEONLY 	0x00000800
#define	MODE_NOPRIVMSGS 	0x00001000
#define	MODE_KEY	      	0x00002000
#define	MODE_BAN	      	0x00004000
#define	MODE_LIMIT	    	0x00008000
#define MODE_REGISTERED		0x00010000
#define MODE_REGONLY		0x00020000
#define MODE_NOCOLOR		0x00040000
#define MODE_OPERONLY   	0x00080000
#define MODE_EXEMPT		0x00100000
#define MODE_NOINVITE		0x00200000
#define MODE_NOKNOCK		0x00400000

/* the hell is this? - lucas */
/* #define	MODE_FLAGS	    0x3fff*/

/* mode flags which take another parameter (With PARAmeterS) */

#define	MODE_WPARAS	(MODE_CHANOP|MODE_VOICE|MODE_BAN|MODE_EXEMPT|MODE_KEY|MODE_LIMIT|MODE_HALFOP|MODE_CHANADMIN)

/*
 * Undefined here, these are used in conjunction with the above modes
 * in the source. #define       MODE_DEL       0x40000000 #define
 * MODE_ADD       0x80000000
 */

#define	HoldChannel(x)		(!(x))

/*name invisible */

#define	SecretChannel(x)	((x) && ((x)->mode.mode & MODE_SECRET))

/* channel not shown but names are */

#define	HiddenChannel(x)	((x) && ((x)->mode.mode & MODE_PRIVATE))

/* channel visible */

#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
#define	PubChannel(x)		((!x) || ((x)->mode.mode &(MODE_PRIVATE | MODE_SECRET)) == 0)

/* #define IsMember(user,chan) 
 * (find_user_link((chan)->members,user) ? 1 : 0) */

#define IsMember(blah,chan) ((blah && blah->user && find_channel_link((blah->user)->channel, chan)) ? 1 : 0)

#define	IsChannelName(name) ((name) && (*(name) == '#' || *(name) == '&'))

/* Misc macros */

#define	BadPtr(x) (!(x) || (*(x) == '\0'))

#define	isvalid(c) (((c) >= 'A' && (c) < '~') || isdigit(c) || (c) == '-')

#define	MyConnect(x)			((x)->fd >= 0)
#define	MyClient(x)			(MyConnect(x) && IsClient(x))
#define	MyOper(x)			(MyConnect(x) && IsOper(x))

/* String manipulation macros */

/* strncopynt --> strncpyzt to avoid confusion, sematics changed N must
 * be now the number of bytes in the array --msa */

#define	strncpyzt(x, y, N) do{(void)strncpy(x,y,N);x[N-1]='\0';}while(0)
#define	StrEq(x,y)	(!strcmp((x),(y)))

/* used in SetMode() in channel.c and m_umode() in s_msg.c */

#define	MODE_NULL      0
#define	MODE_ADD       0x40000000
#define	MODE_DEL       0x20000000

/* return values for hunt_server() */

#define	HUNTED_NOSUCH	(-1)	/* if the hunted server is not found */
#define	HUNTED_ISME	0	/* if this server should execute the command */
#define	HUNTED_PASS	1	/* if message passed onwards successfully */

/* used when sending to #mask or $mask */

#define	MATCH_SERVER  1
#define	MATCH_HOST    2

/* used for async dns values */

#define	ASYNC_NONE	(-1)
#define	ASYNC_CLIENT	0
#define	ASYNC_CONNECT	1
#define	ASYNC_CONF	2
#define	ASYNC_SERVER	3

/* misc variable externs */

extern char version[128], protoctl[512], umodestring[128], cmodestring[128],
  *creditstext[], *copyrighttext[];
extern char *generation, *creation;

/* misc defines */

#define ZIP_NEXT_BUFFER -4
#define RC4_NEXT_BUFFER -3
#define	FLUSH_BUFFER	-2
#define	UTMP		"/etc/utmp"
#define	COMMA		","

#ifdef ORATIMING
/*
 * Timing stuff (for performance measurements): compile with
 * -DORATIMING and put a TMRESET where you want the counter of time
 * spent set to 0, a TMPRINT where you want the accumulated results,
 * and TMYES/TMNO pairs around the parts you want timed -orabidoo
 */

extern struct timeval tsdnow, tsdthen;
extern unsigned long tsdms;

#define TMRESET tsdms=0;
#define TMYES gettimeofday(&tsdthen, NULL);
#define TMNO gettimeofday(&tsdnow, NULL); if (tsdnow.tv_sec!=tsdthen.tv_sec) tsdms+=1000000*(tsdnow.tv_sec-tsdthen.tv_sec); tsdms+=tsdnow.tv_usec; tsdms-=tsdthen.tv_usec;
#define TMPRINT sendto_ops("Time spent: %ld ms", tsdms);
#else
#define TMRESET
#define TMYES
#define TMNO
#define TMPRINT
#endif

/* allow 5 minutes after server rejoins the network before allowing
 * chanops new channel */

#ifdef NO_CHANOPS_WHEN_SPLIT
#define MAX_SERVER_SPLIT_RECOVERY_TIME 5
#endif

#ifdef FLUD
struct fludbot
{
  struct Client *fluder;
  int count;
  time_t first_msg, last_msg;
  struct fludbot *next;
};

#endif /* FLUD */

struct Watch
{
  aWatch *hnext;
  time_t lasttime;
  Link *watch;
  char nick[1];
};

struct ListOptions
{
  LOpts *next;
  Link *yeslist, *nolist;
  int starthash;
  short int showall;
  unsigned short usermin;
  int usermax;
  time_t currenttime;
  time_t chantimemin;
  time_t chantimemax;
  time_t topictimemin;
  time_t topictimemax;
};

typedef struct SearchOptions
{
  int umodes;
  char *nick;
  char *user;
  char *host;
  char *gcos;
  char *ip;
  char *vhost;
  aChannel *channel;
  aClient *server;
  char umode_plus:1;
  char nick_plus:1;
  char user_plus:1;
  char host_plus:1;
  char gcos_plus:1;
  char ip_plus:1;
  char vhost_plus:1;
  char chan_plus:1;
  char serv_plus:1;
  char away_plus:1;
  char check_away:1;
  char check_umode:1;
  char show_chan:1;
  char search_chan:1;
  char isoper:1;
  char spare:4;			/* spare space for more stuff(?) */
}
SOpts;


struct t_crline
{
  char *channel;
  int type;
  aCRline *next, *prev;
};

#define IsSendable(x)      (DBufLength(&x->sendQ) < 16384)
#define DoList(x)    (((x)->user) && ((x)->user->lopt))

/* internal defines for cptr->sockerr */
#define IRCERR_BUFALLOC		-11
#define IRCERR_ZIP		-12

#endif /* __struct_include__ */
