/*
 *  UltimateIRCd - an Internet Relay Chat Daemon, src/s_socks.c
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: s_socks.c,v 1.7 2002/03/09 23:28:18 shadowmaster Exp $
 *
 */

/*
** Based on s_auth.c by Darren Reed (C) 1992,
** s_socks.c by Lucas Madar (C) 1998 and s_socks.c by
** ChatNet Coding Team.
*/


/* All these includes are copied from auth.c because we are
   just doing something very similar */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "res.h"
#include "numeric.h"
#include "version.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#if defined(__hpux)
#include "inet.h"
#endif
#include <fcntl.h>
#include "sock.h"		/*
				 * If FD_ZERO isn't define up to this point,  
				 */
/*
 * define it (BSD4.2 needs this) 
 */
#include "h.h"
#include "fdlist.h"

#ifdef SOCKSPORT


//static unsigned char socksid[12];
#define HICHAR(s)     (((unsigned short) s) >> 8)
#define LOCHAR(s)     (((unsigned short) s) & 0xFF)
#define LISTEN_SIZE 5

/* init_socks
 * Set up socksid, simply.
 */

void
init_proxy (aClient * cptr)
{
  /*
     * Or not - ShadowMaster
   */
  return;
}

/*
 * start_proxy
 *
 * attempts to connect to a socks4 proxy server on port 1080 of the host.
 * if the connect fails, the user passes the proxy check
 * otherwise, the connection is checked to see if it is a "secure"
 * socks4+ server
 */
void
start_proxy (aClient * cptr)
{
  struct sockaddr_in sock;
  struct sockaddr_in localaddr;
  int locallen;

  Debug ((DEBUG_NOTICE, "start_proxy(%x) fd %d status %d",
	  cptr, cptr->fd, cptr->status));
  if ((cptr->proxyfd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
#ifdef	USE_SYSLOG
      syslog (LOG_ERR, "Unable to create proxy check socket for %s:%m",
	      get_client_name (cptr, TRUE));
#endif
      return;
    }
  if (cptr->proxyfd >= MAXCONNECTIONS)
    {
      sendto_ops ("Can't allocate fd for proxy check on %s",
		  get_client_name (cptr, (IsServer (cptr) ? HIDEME : TRUE)));
      (void) close (cptr->proxyfd);
      return;
    }
#ifdef SHOW_HEADERS
  send (cptr->fd, REPORT_DO_PROXY, R_do_proxy, 0);
#endif

  set_non_blocking (cptr->proxyfd, cptr);
  /*
   * get the local address of the client and bind to that to make the
   * proxy request.  This used to be done only for ifdef VIRTTUAL_HOST,
   * but needs to be done for all clients since the ident request must
   * originate from that same address-- and machines with multiple IP
   * addresses are common now
   */
  locallen = sizeof (struct sockaddr_in);

  memset (&localaddr, '\0', locallen);
  getsockname (cptr->fd, (struct sockaddr *) &localaddr, &locallen);
  localaddr.sin_port = htons (0);

  if (bind (cptr->proxyfd, (struct sockaddr *) &localaddr,
	    sizeof (localaddr)) == -1)
    {
      report_error ("binding proxy stream socket %s:%s", cptr);
      (void) close (cptr->proxyfd);
      return;
    }

  memcpy ((char *) &sock.sin_addr, (char *) &cptr->ip,
	  sizeof (struct in_addr));

  sock.sin_port = htons (1080);
  sock.sin_family = AF_INET;

  /*
   * (void)alarm((unsigned)4); 
   */
  if (connect (cptr->proxyfd, (struct sockaddr *) &sock,
	       sizeof (sock)) == -1 && errno != EINPROGRESS)
    {
      /*
       * No error report from this...
       */
      /*
       * (void)alarm((unsigned)0); 
       */
      (void) close (cptr->proxyfd);
      cptr->proxyfd = -1;
#ifdef SHOW_HEADERS
      send (cptr->fd, REPORT_NO_PROXY, R_no_proxy, 0);
#endif
      return;
    }
  /*
   * (void)alarm((unsigned)0); 
   */
  cptr->flags2 |= (FLAGS2_WRPROXY | FLAGS2_PROXY);
  if (cptr->proxyfd > highest_fd)
    highest_fd = cptr->proxyfd;
  return;
}


/*
 * send_proxyquery
 *
 * send the proxy server a query to see if it's open.
 */
void
send_proxyquery (aClient * cptr)
{
  struct sockaddr_in us, them;
  char proxybuf[12];
  int ulen, tlen;
  unsigned long theip;

  Debug ((DEBUG_NOTICE, "write_proxyports(%x) fd %d proxyfd %d stat %d",
	  cptr, cptr->fd, cptr->proxyfd, cptr->status));
  tlen = ulen = sizeof (us);

  if (getsockname (cptr->fd, (struct sockaddr *) &us, &ulen) ||
      getpeername (cptr->fd, (struct sockaddr *) &them, &tlen))
    {
#ifdef	USE_SYSLOG
      syslog (LOG_DEBUG, "proxy get{sock,peer}name error for %s:%m",
	      get_client_name (cptr, TRUE));
#endif
      (void) close (cptr->proxyfd);
      if (cptr->proxyfd == highest_fd)
	while (!local[highest_fd])
	  highest_fd--;
      cptr->proxyfd = -1;
      cptr->flags2 &= ~(FLAGS2_PROXY | FLAGS2_WRPROXY);
      return;
    }

  theip = htonl (us.sin_addr.s_addr);

  proxybuf[0] = 4;		// socks 4
  proxybuf[1] = 1;		// connect request
  proxybuf[2] = ((((unsigned short) ntohs (us.sin_port)) >> 8) & 0xFF);	// hiword of port
  proxybuf[3] = (((unsigned short) ntohs (us.sin_port)) & 0xFF);	// loword of port
  proxybuf[4] = (theip >> 24) & 0xFF;
  proxybuf[5] = (theip >> 16) & 0xFF;
  proxybuf[6] = (theip >> 8) & 0xFF;
  proxybuf[7] = (theip & 0xFF);
  proxybuf[8] = 0;

  Debug ((DEBUG_SEND, "sending [%s] to proxy port %s.1080",
	  proxybuf, inetntoa ((char *) &them.sin_addr)));


  if (write (cptr->proxyfd, proxybuf, strlen (proxybuf)) != strlen (proxybuf))
    {
      (void) close (cptr->proxyfd);
      if (cptr->proxyfd == highest_fd)
	while (!local[highest_fd])
	  highest_fd--;
      cptr->proxyfd = -1;
      cptr->flags2 &= ~FLAGS2_PROXY;
      send (cptr->fd, REPORT_NO_PROXY, R_no_proxy, 0);
      return;
    }

  cptr->flags2 &= ~FLAGS2_WRPROXY;

  return;
}

/*
 * read_proxy
 *
 * process the proxy reply.
 */

void
read_proxy (aClient * cptr)
{
  unsigned char proxybuf[12];
  int len;

  len = recv (cptr->proxyfd, proxybuf, 9, 0);

  Debug ((DEBUG_NOTICE, "read_proxy: %s", proxybuf));

  (void) close (cptr->proxyfd);
  if (cptr->proxyfd == highest_fd)
    while (!local[highest_fd])
      highest_fd--;
  cptr->proxyfd = -1;
  ClearProxy (cptr);

  if (len < 4)
    {
      send (cptr->fd, REPORT_NO_PROXY, R_no_proxy, 0);
      return;
    }
  if (proxybuf[1] == 90)
    {
      cptr->flags2 |= FLAGS2_GOTPROXY;
      return;
    }
  send (cptr->fd, REPORT_GOOD_PROXY, R_good_proxy, 0);
  return;
}

#endif /* SOCKSPORT */
